Attribute VB_Name = "GraphFun"
Option Explicit

Public Declare Function CreateFontIndirect Lib "gdi32" Alias "CreateFontIndirectA" (lpLogFont As LOGFONT) As Long
Public Declare Function SelectObject Lib "gdi32" (ByVal hdc As Long, ByVal hObject As Long) As Long
Public Declare Function TextOut Lib "gdi32" Alias "TextOutA" (ByVal hdc As Long, ByVal x As Long, ByVal Y As Long, ByVal lpString As String, ByVal nCount As Long) As Long
Public Declare Function DeleteObject Lib "gdi32" (ByVal hObject As Long) As Long
Public Declare Function SetTextAlign Lib "gdi32" (ByVal hdc As Long, ByVal wFlags As Long) As Long
Public Declare Function GetTextAlign Lib "gdi32" (ByVal hdc As Long) As Long
Public Const TA_BASELINE = 24
Public Const TA_BOTTOM = 8
Public Const TA_TOP = 0
Public Const TA_CENTER = 6
Public Const TA_LEFT = 0
Public Const TA_RIGHT = 2
Public Const TA_NOUPDATECP = 0
Public Const TA_UPDATECP = 1
Public Declare Function GetDeviceCaps Lib "gdi32" (ByVal hdc As Long, ByVal nIndex As Long) As Long
Public Const LOGPIXELSY = 90        '  Logical pixels/inch in Y
Public Type LOGFONT
        lfHeight As Long
        lfWidth As Long
        lfEscapement As Long
        lfOrientation As Long
        lfWeight As Long
        lfItalic As Byte
        lfUnderline As Byte
        lfStrikeOut As Byte
        lfCharSet As Byte
        lfOutPrecision As Byte
        lfClipPrecision As Byte
        lfQuality As Byte
        lfPitchAndFamily As Byte
        lfFaceName(1 To 32) As Byte
End Type


Public Sub PlotText(msg As String, where As Object, x As Single, Y As Single, Alignement As Integer)
Dim w As Single, h As Single
Dim fx As Single, fy As Single
    w = where.TextWidth(msg)     ' Calculate width
    h = where.TextHeight(msg)    ' Calculate height
    Select Case Alignement
    Case flexAlignLeftTop: fx = 0: fy = 0
    Case flexAlignLeftCenter: fx = 0: fy = 0.5
    Case flexAlignLeftBottom: fx = 0: fy = 1
    Case flexAlignCenterTop: fx = 0.5: fy = 0
    Case flexAlignCenterCenter: fx = 0.5: fy = 0.5
    Case flexAlignCenterBottom: fx = 0.5: fy = 1
    Case flexAlignRightTop: fx = 1: fy = 0
    Case flexAlignRightCenter: fx = 1: fy = 0.5
    Case flexAlignRightBottom: fx = 1: fy = 1
    End Select
       
    where.CurrentX = x - w * fx
    where.CurrentY = Y - h * fy
    where.Print msg   ' Print message.
End Sub

Public Sub OblicText(msg As String, angle, where As Object, Xpos, Ypos, Alignement, FontName As String, FontSize, bold As Boolean, italic As Boolean, underline As Boolean)
Dim lf As LOGFONT
Dim hFont As Long
Dim hPreviousFont As Long
Dim hPic As Long
Dim previousAlign As Long
Dim res
    If Len(msg) = 0 Then Exit Sub
    hPic = where.hdc
    With lf
        .lfHeight = -(FontSize * GetDeviceCaps(hPic, LOGPIXELSY) / 72)
'        .lfHeight = -8
        .lfWidth = 0
        .lfEscapement = angle
        .lfOrientation = angle
        If bold Then .lfWeight = 700 Else .lfWeight = 400
        .lfItalic = italic
        .lfUnderline = underline
        .lfStrikeOut = False
        .lfCharSet = 0
        .lfOutPrecision = 0
        .lfClipPrecision = 0
        .lfQuality = 0
        .lfPitchAndFamily = 32 ' FF_SWISS
        Call NullTerminated(.lfFaceName, FontName)
    End With
    hFont = CreateFontIndirect(lf)
    hPreviousFont = SelectObject(hPic, hFont)
    previousAlign = GetTextAlign(hPic)
    res = SetTextAlign(hPic, Alignement)
    res = TextOut(hPic, Xpos, Ypos, msg, Len(msg))
    res = SelectObject(hPic, hPreviousFont)
    res = SetTextAlign(hPic, previousAlign)
    res = DeleteObject(hFont)

End Sub

Function NullTerminated(ByteArray() As Byte, Text As String)
Dim i As Integer, p As Integer
    p = LBound(ByteArray)
    For i = 1 To Len(Text)
        If p = UBound(ByteArray) Then Exit For
        ByteArray(p) = Asc(Mid(Text, i, 1))
        p = p + 1
    Next i
    ByteArray(p) = 0
End Function

Sub FindTick(Xmin As Double, Xmax As Double, ExtendTicks As Boolean, ByVal nbWanted As Integer, Ticks() As Double, nbReturned As Integer, Optional WithLimits As Boolean = False)
Dim L10 As Double, a As Integer, b As Integer, pas As Double, diff As Double
Dim x As Double
diff = Xmax - Xmin
L10 = Log10(diff)
a = Int(L10)
If (L10 - a) < Log10(2.5) Then a = a - 1
Select Case diff / (10 ^ a)
Case 0 To 4: b = 1
Case 4 To 9: b = 2
Case 9 To 30: b = 5
End Select
nbReturned = 0
pas = b * 10 ^ a
x = pas * Int(Xmin / pas)
While x < Xmax + pas And nbReturned < UBound(Ticks)
    If x > Xmin And x < Xmax Or ExtendTicks Or WithLimits Then
        nbReturned = nbReturned + 1
        Ticks(nbReturned) = x
    End If
    x = x + pas
Wend
End Sub

Function Log10(x) As Double
    Log10 = Log(x) / Log(10#)
End Function


