Attribute VB_Name = "Module1"
Option Explicit

Public Declare Sub Sedtrans05 Lib "sedtrans05.dll" _
    Alias "sedtrans05dll_" (in1 As Double, CONC As Double, _
    BEDCHA As Double, in2 As Double, out1 As Double, out2 As _
    Double, OUT3 As Double, PROFILE As Double, ByVal BEDTXTall As String)

Public Type type_data
    in1(1 To 17) As Double  '  Input variables to Sedtrans
'    IN2(1 To 28) As Double  '  Default parameters for cohesive algorithm
'    CONC(1 To 21) As Double  ' Cohesive SSC
'    BEDCHA(20, 3) As Double  ' Description of cohesive bed
    out1(1 To 16) As Double  ' Results (similar to file *.dat)
    out2(1 To 26) As Double  ' Results (similar to file *.da2)
    OUT3(1 To 5) As Double   ' Results (similar to file *.coh)
    PROFILE(1 To 12, 1 To 3) As Double 'SSC and velocity profile (non-cohesive)
    BEDTXT(1 To 7) As String      ' Bedform predictions
    Calculated As Boolean
    IRUN As Long
End Type

Public Const BatchFile = "sed05.exe"
Public MaxNbRun As Integer
Public Data() As type_data
Public PosData As Integer
Public last_PosData As Integer
Public last_IRUN As Long
Public InputModified As Boolean
Public BatchArgumentFiles(2) As String  ' value of tFile(0-2) in the Batch_form
'Public Const maxdate = 38929

Function Check_existence_sedtrans(mode As Integer) As Boolean
Const DllFile = "sedtrans05.dll"
Dim FullName As String
    If mode = 1 Then
        FullName = App.Path & "\" & DllFile
'        If CDbl(Date) > maxdate Then _
'            MsgBox "This test version of Sedtrans05 is outdated !" & vbNewLine & vbNewLine & "Use a newer version.", vbCritical + vbOKOnly
    Else
        FullName = App.Path & "\" & BatchFile
    End If
    If Dir(FullName) = "" Then
        If mode = 1 Then
            MsgBox DllFile & " was not found." & vbNewLine & vbNewLine & _
                "The DLL-file  " & DllFile & _
                " must be in the directory " & App.Path & ".", _
                vbOKOnly + vbCritical, "Sedtrans04"
        Else
            MsgBox BatchFile & " was not found." & vbNewLine & vbNewLine & _
                "The batch-program " & BatchFile & _
                " must be in the directory " & App.Path & ".", _
                vbOKOnly + vbCritical, "Sedtrans04"
        End If
        Check_existence_sedtrans = False
    Else
        Check_existence_sedtrans = True
    End If
End Function


Sub ReadInput(in1() As Double)
Dim i As Integer
    For i = 1 To 7
        If Fmain.Calmethod(i).value Then
            Fmain.inputval(1) = i
            Exit For
        End If
    Next i
    For i = 1 To 15
        in1(i) = CDbl(Fmain.inputval(i))
    Next i
    in1(9) = in1(9) / 1000

End Sub


Sub Do_Calculations()
Dim BEDTXTall As String * 560
Dim i As Integer
'    If CDbl(Date) > maxdate Then Exit Sub

    If Data(PosData).Calculated Then Exit Sub

    Call ReadInput(Data(PosData).in1)
       
    Call Sedtrans05(Data(PosData).in1(1), 0, 0, 0, Data(PosData).out1(1), _
        Data(PosData).out2(1), Data(PosData).OUT3(1), _
        Data(PosData).PROFILE(1, 1), BEDTXTall)
        
    For i = 1 To 7
        Data(PosData).BEDTXT(i) = Trim(Mid(BEDTXTall, i * 80 - 79, 80))
    Next i
    
    Data(PosData).Calculated = True
    
    Call show_results
    InputModified = False

End Sub


Sub show_input()
Dim i As Integer
    For i = 1 To 15
        Fmain.inputval(i) = Data(PosData).in1(i)
    Next i
    Fmain.inputval(9) = Data(PosData).in1(9) * 1000
    Fmain.l_irun = Data(PosData).IRUN
    Fmain.Calmethod(Data(PosData).in1(1)) = True
End Sub


Sub show_results()
Dim TimeNoTransport, ch As String, i As Integer
If Data(PosData).Calculated Then
    Fmain.l_ResultsTitle(1) = Data(PosData).IRUN
    Fmain.l_ResultsValue(0) = FormatN(Data(PosData).out1(15), 7, 4)
    Fmain.l_ResultsValue(1) = FormatN(Data(PosData).out1(14), 7, 4)
    Fmain.l_ResultsValue(2) = Format(Data(PosData).out1(16), "0.0") & " "
    Fmain.l_ResultsValue(3) = Format(Data(PosData).out2(21), "0.0") & " %"
    Fmain.l_ResultsValue(4) = Format(Data(PosData).out2(22), "0.0") & " %"
    TimeNoTransport = 100 - Data(PosData).out2(21) - Data(PosData).out2(22)
    If TimeNoTransport < 0 Then TimeNoTransport = 0
    If TimeNoTransport > 100 Then TimeNoTransport = 100
    Fmain.l_ResultsValue(5) = Format(TimeNoTransport, "0.0") & " %"
    Call FillOutGrid(False)
    ch = "EXPECTED BEDFORMS ARE (Amos, 1990; Li & Amos, 1998):" & vbNewLine
    For i = 1 To 7
        If Data(PosData).BEDTXT(i) <> "" Then _
            ch = ch & Data(PosData).BEDTXT(i) & vbNewLine
    Next i
    Fmain.t_bedforms = ch
    ShowValidity (True)
Else
    Fmain.l_ResultsTitle(1) = "---"
    Fmain.l_ResultsValue(0) = "---       "
    Fmain.l_ResultsValue(1) = "---       "
    Fmain.l_ResultsValue(2) = "---"
    Fmain.l_ResultsValue(3) = "---       "
    Fmain.l_ResultsValue(4) = "---       "
    Fmain.l_ResultsValue(5) = "---       "
    Call FillOutGrid(True)
    Fmain.t_bedforms = "EXPECTED BEDFORMS ARE (Amos, 1990; Li & Amos, 1998):"
    ShowValidity (False)
End If
End Sub




Sub SaveAllData(only_input As Boolean)
' only_input=true  save only input
' only_input=false   save input + output
Dim fileID As Integer, i As Integer, j As Integer, k As Integer, ready As Boolean
Dim File_fullname As String
Dim ext As String, name As String, ch As String, rep

    With Fmain.CommonDialog1
        If only_input Then
            .DialogTitle = "Save all input data in batch-file format"
            .Flags = cdlOFNOverwritePrompt Or cdlOFNHideReadOnly Or cdlOFNNoReadOnlyReturn
            ' Prompt for Overwrite, No ReadOnly check box, No readonly file,
            ready = True
        Else
            .DialogTitle = "Name (without extension) for input file and several output files"
            .Flags = cdlOFNHideReadOnly Or cdlOFNNoReadOnlyReturn
            ' No ReadOnly check box, No readonly file,
        End If
        .DefaultExt = "dat"
        .Filter = "dat files (*.dat)|*.dat|All files (*.*)|*.*"
        .FilterIndex = 1
        .CancelError = True
        .FileName = ""
        
        Do
        On Error GoTo cancelled
            .ShowSave
        On Error GoTo 0
            If Not only_input Then
                name = .FileName
                ext = FilenameExtension(name)
                If ext <> "" Then
                    name = Left(name, Len(name) - 1 - Len(ext))
                End If
                If Dir(name & ".dat") <> "" Or Dir(name & ".da2") <> "" Or _
                    Dir(name & ".pro") <> "" Or Dir(name & ".in") <> "" Then
                    ch = FilenameShort(name)
                    rep = MsgBox("One or several of the files " & ch & ".in, " & ch & ".dat, " & ch & ".da2, " & ch & ".pro already exist and will be overwritten." & vbNewLine & vbNewLine & "Do you want to continue ?", vbExclamation + vbOKCancel)
                    ready = (rep <> vbCancel)
                Else
                    ready = True
                End If
            End If
        Loop Until ready

        If .FileName = "" Then
cancelled:  Exit Sub
        Else
            File_fullname = .FileName
        End If
    End With
    
    On Error GoTo ErrIndata
    
    fileID = FreeFile
    If Not only_input Then File_fullname = name & ".in"
    Open File_fullname For Output Lock Read Write As #fileID
    Print #fileID, "IRUN" & vbTab & "IOPT1" & vbTab & "D" & vbTab & "UZ" & _
        vbTab & "Z" & vbTab & "CDIR" & vbTab & "HT" & vbTab & "PER" & vbTab _
        & "WDIR" & vbTab & "GD" & vbTab & "RHINP" & vbTab & "RLINP" & vbTab _
        & "BETA" & vbTab & "RHOS" & vbTab & "SALIN" & vbTab & "TEMP"
    For i = 1 To last_PosData
        Print #fileID, Data(i).IRUN;
        For j = 1 To 15
            Print #fileID, vbTab & Data(i).in1(j);
        Next j
        Print #fileID,
    Next i
    Close #fileID
    If only_input Then Exit Sub
    
    File_fullname = name & ".dat"
    Open File_fullname For Output Lock Read Write As #fileID
    Print #fileID, "IRUN" & vbTab & "ERR" & vbTab & "UB" & vbTab & "FCW" & _
        vbTab & "UA" & vbTab & "U100" & vbTab & "USTCWS" & vbTab & "USTCW" _
        & vbTab & "Z0" & vbTab & "FALL" & vbTab & "RHEIGHT" & vbTab & _
        "RLENGTH" & vbTab & "C0A" & vbTab & "QS" & vbTab & "QSDIR" & vbTab _
        & "SEDM" & vbTab & "SED" & vbTab & "SEDDIR"
    For i = 1 To last_PosData
        Print #fileID, Data(i).IRUN & vbTab & "0";
        For j = 1 To 16
            Print #fileID, vbTab & Data(i).out1(j);
        Next j
        Print #fileID,
    Next i
    Close #fileID
    
    File_fullname = name & ".da2"
    Open File_fullname For Output Lock Read Write As #fileID
    Print #fileID, "IRUN" & vbTab & "AB" & vbTab & "WL" & _
        vbTab & "USTCRB" & vbTab & "USTCRS" & vbTab & "USTUP" & vbTab & _
        "Z0C" & vbTab & "PHIB" & vbTab & "PHI100" & vbTab & "DELTACW" & _
        vbTab & "USTCS" & vbTab & "USTWS" & vbTab & "USTCWSE" & vbTab & _
        "USTCWSB" & vbTab & "USTC" & vbTab & "USTW" & vbTab & "RPLCOEF" & _
        vbTab & "TB1" & vbTab & "TB2" & vbTab & "TS1" & vbTab & "TS2" & _
        vbTab & "PERBED" & vbTab & "PERSUSP" & vbTab & "C0" & vbTab & _
        "TAOCWS" & vbTab & vbTab & "RHOW" & vbTab & "VISC"
    For i = 1 To last_PosData
        Print #fileID, Data(i).IRUN;
        For j = 1 To 26
            Print #fileID, vbTab & Data(i).out2(j);
        Next j
        Print #fileID,
    Next i
    Close #fileID
    
    File_fullname = name & ".pro"
    Open File_fullname For Output Lock Read Write As #fileID
    Print #fileID, "IRUN" & vbTab & "NB_Z" & vbTab & "z1" & vbTab & "z2" _
        & vbTab & "z3" & vbTab & "z4" & vbTab & "z5" & vbTab & _
        "z6" & vbTab & "z7" & vbTab & "z8" & vbTab & "z9" & vbTab & "z10" _
        & vbTab & "z11" & vbTab & "z12" & vbTab & "C1" & vbTab & "C2" _
        & vbTab & "C3" & vbTab & "C4" & vbTab & "C5" & vbTab & "C6" & _
        vbTab & "C7" & vbTab & "C8" & vbTab & "C9" & vbTab & "C10" & _
        vbTab & "C11" & vbTab & "C12" & vbTab & "U1" & vbTab & "U2" & _
        vbTab & "U3" & vbTab & "U4" & vbTab & "U5" & vbTab & "U6" & _
        vbTab & "U7" & vbTab & "U8" & vbTab & _
        "U9" & vbTab & "U10" & vbTab & "U11" & vbTab & "U12"
    For i = 1 To last_PosData
        Print #fileID, Data(i).IRUN & vbTab & "12";
        For j = 1 To 3
            For k = 1 To 12
                Print #fileID, vbTab & Data(i).PROFILE(k, j);
            Next k
        Next j
        Print #fileID,
    Next i
    Close #fileID
    
    ext = vbNewLine & "          "
    MsgBox "The data were saved in the files:" & ext & ch & ".in" _
        & ext & ch & ".dat" & ext & ch & ".da2" _
        & ext & ch & ".pro", vbOKOnly
    Exit Sub

ErrIndata:
    MsgBox "Error during writing to the file " & File_fullname, vbOKOnly + vbCritical, "Sedtrans05"
    On Error GoTo ErrIndata2
    Close #fileID
ErrIndata2:
End Sub


Sub ShowValidity(ResultsValid As Boolean)
Dim i As Integer
If ResultsValid Then
    For i = 0 To 5
        Fmain.l_ResultsValue(i).ForeColor = &H80000012
    Next i
    For i = 0 To 2
'        Fmain.l_ResultsText(i).FontBold = True
        Fmain.l_ResultsValue(i).FontBold = True
        Fmain.l_ResultsUnits(i).FontBold = True
    Next i
'    Fmain.l_ResultsTitle(0).FontBold = True
    Fmain.l_ResultsTitle(1).FontBold = True
    Fmain.l_ResultsTitle(1).ForeColor = &H80000012
    Fmain.OutGrid.ForeColor = &H80000008
Else
    For i = 0 To 5
        Fmain.l_ResultsValue(i).ForeColor = &H80000011
    Next i
    For i = 0 To 2
'        Fmain.l_ResultsText(i).FontBold = False
        Fmain.l_ResultsValue(i).FontBold = False
        Fmain.l_ResultsUnits(i).FontBold = False
    Next i
'    Fmain.l_ResultsTitle(0).FontBold = False
    Fmain.l_ResultsTitle(1).FontBold = False
    Fmain.l_ResultsTitle(1).ForeColor = &H80000011
    Fmain.OutGrid.ForeColor = &H80000011
End If
End Sub


Sub PrepareNextRun()
Dim i As Integer, diff As Integer

If PosData = 0 Then   ' Start of program
    last_PosData = 0
    last_IRUN = 0
    MaxNbRun = 100
    ReDim Data(1 To MaxNbRun) As type_data
ElseIf Not Data(last_PosData).Calculated Then
    PosData = last_PosData
    Call ReadInput(Data(PosData).in1)
    Call ShowValidity(False)
    Fmain.l_irun = Data(PosData).IRUN
    InputModified = True
    Fmain.b_PrevNext(1).Enabled = False
    Exit Sub
ElseIf last_PosData = MaxNbRun Then
    diff = 5
    If MaxNbRun - diff < 1 Then diff = MaxNbRun - 1
    For i = 1 To MaxNbRun - diff
        Data(i) = Data(i + diff)
    Next i
    last_PosData = last_PosData - diff
End If

last_PosData = last_PosData + 1
last_IRUN = last_IRUN + 1
PosData = last_PosData
Data(PosData).IRUN = last_IRUN
Fmain.l_irun = Data(PosData).IRUN
Call ReadInput(Data(PosData).in1)
InputModified = True
Data(PosData).Calculated = False
ShowValidity (False)
If PosData > 1 Then Fmain.b_PrevNext(0).Enabled = True
Fmain.b_PrevNext(1).Enabled = False
    
End Sub


Sub Show_PopUpMenu2()
    If Left(Fmain.l_ResultsValue(1), 3) <> "---" Then
        Fmain.PopupMenu Fmain.m_edit2, vbPopupMenuRightButton
    End If
End Sub


' Check the validity of a input value
' iv is the value
' index is the parameter position in the input array Fmain.inputval
' If an error is found, an error message is return,
' otherwise an empty string is returned
Public Function Check_inputval(iv As Double, Index As Integer) As String
Dim msg As String
msg = ""
If iv < 0 And Index <> 12 Then
    msg = "The value must not be lower than zero"
ElseIf iv = 0 And (Index = 2 Or Index = 4 Or Index = 9) Then
    msg = "The value must not be zero"
Else
    Select Case Index
    Case 5, 8
        If iv > 360 Then
            msg = "The value must be between 0 and 360"
        End If
    Case 2, 4, 6, 10
        If CDbl(Fmain.inputval(4)) > CDbl(Fmain.inputval(2)) Then
            msg = "The current height must be smaller than the water depth"
        ElseIf CDbl(Fmain.inputval(10)) > CDbl(Fmain.inputval(2)) Then
            msg = "The ripple height must be smaller than the water depth"
        ElseIf CDbl(Fmain.inputval(6)) >= CDbl(Fmain.inputval(2)) Then
            msg = "The wave height must be smaller than the water depth"
        End If
    Case 9
        If iv > 256 Or iv < 0.01 Then
            msg = "the value must be between 0.01 and 256 mm"
        End If
    Case 12
        If Abs(iv) > 35 Then
            msg = "The value must be between -35 and 35"
        End If
    Case 13
        If iv < 1030 Or iv > 22000 Then
            msg = "The value must be between 1030 and 22000"
        End If
    Case 14
        If iv < 0 Or iv > 42 Then
            msg = "The value must be between 0 and 42"
        End If
    Case 15
        If iv <= 0 Or iv > 35 Then
            msg = "The value must be between 0C and 35C"
        End If
    End Select
End If
Check_inputval = msg
End Function

